@@ -1,8 +1,9 @@
-/*
- * gtksourcecompletionprovider.c
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*- */
+/* gtksourcecompletionprovider.c
  * This file is part of gtksourcecompletion
  *
  * Copyright (C) 2007 -2009 Jesús Barbero Rodríguez <chuchiperriman@gmail.com>
+ * Copyright (C) 2009 - Jesse van den Kieboom <jessevdk@gnome.org>
  *
  * gtksourceview is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,11 +22,11 @@
 
 
 /**
- * SECTION:gtk_source_completion-provider
+ * SECTION:completionprovider
  * @title: GtkSourceCompletionProvider
  * @short_description: Completion provider interface
  *
- * You must implement this interface to provide proposals to #GscCompletion
+ * You must implement this interface to provide proposals to #GtkSourceCompletion
  * 
  */
 
@@ -96,6 +97,19 @@ gtk_source_completion_provider_activate_proposal_default (GtkSourceCompletionPro
 	return FALSE;
 }
 
+static gint
+gtk_source_completion_provider_get_interactive_delay_default (GtkSourceCompletionProvider *provider)
+{
+	/* -1 means the default value in the completion object */
+	return -1;
+}
+
+static gint
+gtk_source_completion_provider_get_priority_default (GtkSourceCompletionProvider *provider)
+{
+	return 0;
+}
+
 static void 
 gtk_source_completion_provider_base_init (GtkSourceCompletionProviderIface *iface)
 {
@@ -115,6 +129,9 @@ gtk_source_completion_provider_base_init (GtkSourceCompletionProviderIface *ifac
 	iface->get_start_iter = gtk_source_completion_provider_get_start_iter_default;
 	iface->activate_proposal = gtk_source_completion_provider_activate_proposal_default;
 
+	iface->get_interactive_delay = gtk_source_completion_provider_get_interactive_delay_default;
+	iface->get_priority = gtk_source_completion_provider_get_priority_default;
+
 	if (!initialized)
 	{
 		initialized = TRUE;
@@ -343,3 +360,41 @@ gtk_source_completion_provider_activate_proposal (GtkSourceCompletionProvider *p
 	                                                                                   proposal,
 	                                                                                   iter);
 }
+
+/**
+ * gtk_source_completion_provider_get_interactive_delay:
+ * @provider: A # GtkSourceCompletionProvider
+ *
+ * Get the delay in milliseconds before starting interactive completion for
+ * this provider. A value of -1 indicates to use the default value as set
+ * by #GtkSourceCompletion::auto-complete-delay.
+ *
+ * Returns: the interactive delay in milliseconds.
+ *
+ **/
+gint
+gtk_source_completion_provider_get_interactive_delay (GtkSourceCompletionProvider *provider)
+{
+	g_return_val_if_fail (GTK_IS_SOURCE_COMPLETION_PROVIDER (provider), -1);
+
+	return GTK_SOURCE_COMPLETION_PROVIDER_GET_INTERFACE (provider)->get_interactive_delay (provider);
+}
+
+/**
+ * gtk_source_completion_provider_get_priority:
+ * @provider: A # GtkSourceCompletionProvider
+ *
+ * Get the provider priority. The priority determines the order in which
+ * proposals appear in the completion popup. Higher priorities are sorted
+ * before lower priorities. The default priority is 0.
+ *
+ * Returns: the provider priority.
+ *
+ **/
+gint
+gtk_source_completion_provider_get_priority (GtkSourceCompletionProvider *provider)
+{
+	g_return_val_if_fail (GTK_IS_SOURCE_COMPLETION_PROVIDER (provider), 0);
+
+	return GTK_SOURCE_COMPLETION_PROVIDER_GET_INTERFACE (provider)->get_priority (provider);
+}
